@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2011-2014 GRNET S.A.
+# Copyright (C) 2011-2015 GRNET S.A.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,14 +17,19 @@
 
 """Module hosting the Image class."""
 
-from image_creator.util import FatalError, QemuNBD
+from image_creator.util import FatalError, QemuNBD, get_command
 from image_creator.gpt import GPTPartitionTable
 from image_creator.os_type import os_cls
 
+import os
 import re
-import guestfs
 import hashlib
 from sendfile import sendfile
+import threading
+
+# Make sure libguestfs runs qemu directly to launch an appliance.
+os.environ['LIBGUESTFS_BACKEND'] = 'direct'
+import guestfs  # noqa
 
 
 class Image(object):
@@ -56,6 +61,11 @@ class Image(object):
             raise FatalError("qemu-nbd command is missing, only raw input "
                              "media are supported")
 
+        # Check If MOUNT LOCAL is supported for this guestfs build
+        self.mount_local_support = hasattr(self.g, "mount_local")
+        if self.mount_local_support:
+            self._mount_thread = None
+
     def check_guestfs_version(self, major, minor, release):
         """Checks if the version of the used libguestfs is smaller, equal or
         greater than the one specified by the major, minor and release triplet
@@ -79,7 +89,7 @@ class Image(object):
 
         self.enable_guestfs()
 
-        self.out.output('Inspecting Operating System ...', False)
+        self.out.info('Inspecting Operating System ...', False)
         roots = self.g.inspect_os()
 
         if len(roots) == 0 or len(roots) > 1:
@@ -143,7 +153,7 @@ class Image(object):
         # process has opened pipes. Since the recovery process is an optional
         # feature of libguestfs, it's better to disable it.
         if self.check_guestfs_version(1, 17, 14) >= 0:
-            self.out.output("Enabling recovery process ...", False)
+            self.out.info("Enabling recovery process ...", False)
             self.g.set_recovery_proc(1)
             self.out.success('done')
         else:
@@ -152,7 +162,7 @@ class Image(object):
         # self.g.set_trace(1)
         # self.g.set_verbose(1)
 
-        self.out.output('Launching helper VM (may take a while) ...', False)
+        self.out.info('Launching helper VM (may take a while) ...', False)
         # self.progressbar = self.out.Progress(100, "Launching helper VM",
         #                                     "percent")
         # eh = self.g.set_event_callback(self.progress_callback,
@@ -161,7 +171,8 @@ class Image(object):
             self.g.launch()
         except RuntimeError as e:
             raise FatalError(
-                "Launching libguestfs's helper VM failed! Reason: %s" % str(e))
+                "Launching libguestfs's helper VM failed!\nReason: %s.\n\n"
+                "Please run `libguestfs-test-tool' for more info." % str(e))
 
         self.guestfs_enabled = True
         # self.g.delete_event_callback(eh)
@@ -180,7 +191,7 @@ class Image(object):
             self.out.warn("Guestfs is already disabled")
             return
 
-        self.out.output("Shutting down helper VM ...", False)
+        self.out.info("Shutting down helper VM ...", False)
         self.g.sync()
         # guestfs_shutdown which is the preferred way to shutdown the backend
         # process was introduced in version 1.19.16
@@ -225,7 +236,7 @@ class Image(object):
         # Self gets overwritten
         img = self
 
-        class RawImage:
+        class RawImage(object):
             """The RawImage context manager"""
             def __enter__(self):
                 return img.device if img.format == 'raw' else \
@@ -262,18 +273,20 @@ class Image(object):
                 "partition tables are supported" % self.meta['PARTITION_TABLE']
             raise FatalError(msg)
 
-        is_extended = lambda p: \
-            self.g.part_get_mbr_id(self.guestfs_device, p['part_num']) \
-            in (0x5, 0xf)
-        is_logical = lambda p: \
-            self.meta['PARTITION_TABLE'] == 'msdos' and p['part_num'] > 4
+        def is_extended(partition):
+            return self.g.part_get_mbr_id(
+                self.guestfs_device, partition['part_num']) in (0x5, 0xf)
+
+        def is_logical(partition):
+            return self.meta['PARTITION_TABLE'] == 'msdos' and \
+                partition['part_num'] > 4
 
         partitions = self.g.part_list(self.guestfs_device)
         last_partition = partitions[-1]
 
         if is_logical(last_partition):
             # The disk contains extended and logical partitions....
-            extended = filter(is_extended, partitions)[0]
+            extended = [p for p in partitions if is_extended(p)][0]
             last_primary = [p for p in partitions if p['part_num'] <= 4][-1]
 
             # check if extended is the last primary partition
@@ -286,37 +299,59 @@ class Image(object):
         """Shrink the image.
 
         This is accomplished by shrinking the last file system of the
-        image and then updating the partition table. The new disk size
-        (in bytes) is returned.
+        image and then updating the partition table. The shrinked device is
+        returned.
 
-        ATTENTION: make sure unmount is called before shrink
+        ATTENTION: make sure umount is called before shrink
         """
-        get_fstype = lambda p: \
-            self.g.vfs_type("%s%d" % (self.guestfs_device, p['part_num']))
-        is_logical = lambda p: \
-            self.meta['PARTITION_TABLE'] == 'msdos' and p['part_num'] > 4
-        is_extended = lambda p: \
-            self.meta['PARTITION_TABLE'] == 'msdos' and \
-            self.g.part_get_mbr_id(self.guestfs_device, p['part_num']) \
-            in (0x5, 0xf)
+        def get_fstype(partition):
+            """Get file system type"""
+            device = "%s%d" % (self.guestfs_device, partition['part_num'])
+            return self.g.vfs_type(device)
 
-        part_add = lambda ptype, start, stop: \
+        def is_logical(partition):
+            """Returns True if the partition is a logical partition"""
+            return self.meta['PARTITION_TABLE'] == 'msdos' and \
+                partition['part_num'] > 4
+
+        def is_extended(partition):
+            """Returns True if the partition is an extended partition"""
+            if self.meta['PARTITION_TABLE'] == 'msdos':
+                mbr_id = self.g.part_get_mbr_id(self.guestfs_device,
+                                                partition['part_num'])
+                return mbr_id in (0x5, 0xf)
+            return False
+
+        def part_add(ptype, start, stop):
+            """Add partition"""
             self.g.part_add(self.guestfs_device, ptype, start, stop)
-        part_del = lambda p: self.g.part_del(self.guestfs_device, p)
-        part_get_id = lambda p: self.g.part_get_mbr_id(self.guestfs_device, p)
-        part_set_id = lambda p, id: \
-            self.g.part_set_mbr_id(self.guestfs_device, p, id)
-        part_get_bootable = lambda p: \
-            self.g.part_get_bootable(self.guestfs_device, p)
-        part_set_bootable = lambda p, bootable: \
-            self.g.part_set_bootable(self.guestfs_device, p, bootable)
+
+        def part_del(partnum):
+            """Delete a partition"""
+            self.g.part_del(self.guestfs_device, partnum)
+
+        def part_get_id(partnum):
+            """Returns the MBR id of the partition"""
+            return self.g.part_get_mbr_id(self.guestfs_device, partnum)
+
+        def part_set_id(partnum, id):
+            """Sets the MBR id of the partition"""
+            self.g.part_set_mbr_id(self.guestfs_device, partnum, id)
+
+        def part_get_bootable(partnum):
+            """Returns the bootable flag of the partition"""
+            return self.g.part_get_bootable(self.guestfs_device, partnum)
+
+        def part_set_bootable(partnum, bootable):
+            """Sets the bootable flag for a partition"""
+            self.g.part_set_bootable(self.guestfs_device, partnum, bootable)
 
         MB = 2 ** 20
 
         if self.is_unsupported():
             if not silent:
                 self.out.warn("Shrinking is disabled for unsupported images")
-            return self.size
+            return None
 
         sector_size = self.g.blockdev_getss(self.guestfs_device)
 
@@ -346,14 +381,21 @@ class Image(object):
             if not silent:
                 self.out.warn(
                     "Don't know how to shrink %s partitions." % fstype)
-            return self.size
+            return None
 
         part_dev = "%s%d" % (self.guestfs_device, last_part['part_num'])
 
-        if self.check_guestfs_version(1, 15, 17) >= 0:
-            self.g.e2fsck(part_dev, forceall=1)
-        else:
-            self.g.e2fsck_f(part_dev)
+        try:
+            if self.check_guestfs_version(1, 15, 17) >= 0:
+                self.g.e2fsck(part_dev, forceall=1)
+            else:
+                self.g.e2fsck_f(part_dev)
+        except RuntimeError as e:
+            # There is a bug in some versions of libguestfs and a RuntimeError
+            # is thrown although the command has successfully corrected the
+            # found file system errors.
+            if e.message.find('***** FILE SYSTEM WAS MODIFIED *****') == -1:
+                raise
 
         self.g.resize2fs_M(part_dev)
 
@@ -406,7 +448,7 @@ class Image(object):
 
         new_size = (end + 1) * sector_size
 
-        assert (new_size <= self.size)
+        assert new_size <= self.size
 
         if self.meta['PARTITION_TABLE'] == 'gpt':
             with self.raw_device(readonly=False) as raw:
@@ -419,7 +461,72 @@ class Image(object):
             self.out.success("Image size is %dMB" %
                              ((self.size + MB - 1) // MB))
 
-        return self.size
+        return part_dev
+
+    def mount(self, mpoint, readonly=False):
+        """Mount the image file system under a local directory"""
+
+        assert self.mount_local_support, \
+            "MOUNT LOCAL not supported for this build of libguestfs"
+
+        assert self._mount_thread is None, "Image is already mounted"
+
+        self.os.mount(readonly=readonly, silent=True)
+
+        if self.g.mount_local(mpoint, readonly=readonly) == -1:
+            return
+
+        # The thread will block in mount_local_run until the file
+        self._mount_thread = threading.Thread(target=self.g.mount_local_run)
+        self._mount_thread.mpoint = mpoint
+        self._mount_thread.start()
+
+    def is_mounted(self):
+        """Check if the image is mounted"""
+
+        assert self.mount_local_support, \
+            "MOUNT LOCAL not supported for this build of libguestfs"
+
+        if self._mount_thread is None:
+            return False
+
+        # Wait for 0.1 second to avoid race conditions if the thread is in an
+        # initialization state but not alive yet.
+        self._mount_thread.join(0.1)
+
+        return self._mount_thread.is_alive()
+
+    def umount(self, lazy=False):
+        """umount the previously mounted image file system"""
+
+        assert self.mount_local_support, \
+            "MOUNT LOCAL not supported for this build of libguestfs"
+
+        assert self._mount_thread is not None, "Image is not mounted"
+
+        try:
+            # Maybe the image was umounted externally
+            if not self._mount_thread.is_alive():
+                self._mount_thread = None
+                return True
+
+            try:
+                args = (['-l'] if lazy else []) + [self._mount_thread.mpoint]
+                get_command('umount')(*args)
+            except:
+                return False
+
+            # Wait for a little while. If the image is umounted,
+            # mount_local_run should have terminated
+            self._mount_thread.join(5)
+
+            if self._mount_thread.is_alive():
+                raise FatalError('Unable to join the mount thread')
+
+            self._mount_thread = None
+            return True
+        finally:
+            self.os.umount()
 
     def dump(self, outfile):
         """Dumps the content of the image into a file.
