@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2011-2014 GRNET S.A.
+# Copyright (C) 2011-2015 GRNET S.A.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ deployment.
 """
 
 import sys
+import logging
 
 from os.path import basename
 
@@ -37,15 +38,18 @@ except ImportError:
     pass
 
 try:
+    logger = logging.getLogger("kamaki.cli.config")
+    logger.setLevel(logging.ERROR)
     config = Config()
 except Exception as e:
     sys.stderr.write("Kamaki config error: %s\n" % str(e))
     sys.exit(1)
 
+CONTAINER = "images"
+
 
 class Kamaki(object):
     """Wrapper class for the ./kamaki library"""
-    CONTAINER = "images"
 
     @staticmethod
     def get_default_cloud_name():
@@ -130,19 +134,23 @@ class Kamaki(object):
             self.account.get_service_endpoints('object-store')['publicURL'],
             self.account.token,
             self.account.user_info()['id'],
-            self.CONTAINER)
+            CONTAINER)
 
         self.image = ImageClient(
             self.account.get_service_endpoints('image')['publicURL'],
             self.account.token)
 
-    def upload(self, file_obj, size=None, remote_path=None, hp=None, up=None):
+    def upload(self, file_obj, size=None, remote_path=None, container=None,
+               content_type=None, hp=None, up=None):
         """Upload a file to Pithos+"""
 
         path = basename(file_obj.name) if remote_path is None else remote_path
 
+        if container is None:
+            container = CONTAINER
+
         try:
-            self.pithos.create_container(self.CONTAINER)
+            self.pithos.create_container(container)
         except ClientError as e:
             if e.status != 202:  # Ignore container already exists errors
                 raise e
@@ -150,37 +158,43 @@ class Kamaki(object):
         hash_cb = self.out.progress_generator(hp) if hp is not None else None
         upload_cb = self.out.progress_generator(up) if up is not None else None
 
-        self.pithos.upload_object(path, file_obj, size, hash_cb, upload_cb)
+        try:
+            self.pithos.container = container
+            self.pithos.upload_object(path, file_obj, size=size,
+                                      hash_cb=hash_cb, upload_cb=upload_cb,
+                                      content_type=content_type)
+        finally:
+            self.pithos.container = CONTAINER
 
         return "pithos://%s/%s/%s" % (self.account.user_info()['id'],
-                                      self.CONTAINER, path)
+                                      container, path)
 
     def register(self, name, location, metadata, public=False):
         """Register an image with Cyclades"""
 
-        # Convert all metadata to strings
-        str_metadata = {}
-        for (key, value) in metadata.iteritems():
-            str_metadata[str(key)] = str(value)
         is_public = 'true' if public else 'false'
         params = {'is_public': is_public, 'disk_format': 'diskdump'}
-        return self.image.register(name, location, params, str_metadata)
+        return self.image.register(name, location, params, metadata)
 
     def share(self, location):
         """Share this file with all the users"""
 
         self.pithos.set_object_sharing(location, "*")
 
-    def object_exists(self, location):
+    def object_exists(self, container, location):
         """Check if an object exists in Pithos+"""
 
         try:
+            self.pithos.container = container
             self.pithos.get_object_info(location)
         except ClientError as e:
+            self.pithos.container = CONTAINER
             if e.status == 404:  # Object not found error
                 return False
             else:
                 raise
+
+        self.pithos.container = CONTAINER
         return True
 
 # vim: set sta sts=4 shiftwidth=4 sw=4 et ai :
