@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2011-2014 GRNET S.A.
+# Copyright (C) 2011-2015 GRNET S.A.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """This package hosts OS-specific code common for the various Microsoft
-Windows OSs."""
+Windows OSes."""
 
 from image_creator.os_type import OSBase, sysprep, add_sysprep_param
 from image_creator.util import FatalError
@@ -235,8 +235,8 @@ class Windows(OSBase):
     @add_sysprep_param(
         'connection_retries', "posint", 5, DESCR['connection_retries'])
     @add_sysprep_param(
-        'shutdown_timeout', "posint", 120, DESCR['shutdown_timeout'])
-    @add_sysprep_param('boot_timeout', "posint", 300, DESCR['boot_timeout'])
+        'shutdown_timeout', "posint", 300, DESCR['shutdown_timeout'])
+    @add_sysprep_param('boot_timeout', "posint", 600, DESCR['boot_timeout'])
     @add_sysprep_param('virtio', 'dir', "", DESCR['virtio'],
                        check=virtio_dir_check, hidden=True)
     @add_sysprep_param(
@@ -271,7 +271,7 @@ class Windows(OSBase):
         self.registry = Registry(self.image)
 
         with self.mount(readonly=True, silent=True):
-            self.out.output("Checking media state ...", False)
+            self.out.info("Checking media state ...", False)
 
             # Enumerate the windows users
             (self.usernames,
@@ -393,73 +393,17 @@ class Windows(OSBase):
         defragged.
         """
 
-        # Query for the maximum number of reclaimable bytes
-        cmd = (
-            r'cmd /Q /V:ON /C "SET SCRIPT=%TEMP%\QUERYMAX_%RANDOM%.TXT & ' +
-            r'ECHO SELECT DISK 0 > %SCRIPT% & ' +
-            'ECHO SELECT PARTITION %d >> %%SCRIPT%% & ' % self.last_part_num +
-            r'ECHO SHRINK QUERYMAX >> %SCRIPT% & ' +
-            r'ECHO EXIT >> %SCRIPT% & ' +
-            r'DISKPART /S %SCRIPT% & ' +
-            r'IF NOT !ERRORLEVEL! EQU 0 EXIT /B 1 & ' +
-            r'DEL /Q %SCRIPT%"')
-
-        stdout, stderr, rc = self.vm.rexec(cmd)
-
-        querymax = None
-        expr = re.compile(
-            r'.+:\s*(\d+)\s*([KMGT]?)B\s*(?:\((\d+)\s*([KMGT]?)B\))?\s*$')
-        for line in stdout.splitlines():
-            # diskpart will return something like this:
-            #
-            # a) The maximum number of reclaimable bytes is: xxxx MB
-            # b) The maximum number of reclaimable bytes is: xxxx GB (xxxx MB)
-
-            match = expr.match(line)
-            if match:
-                offset = 0 if match.group(3) is None else 2
-                querymax = match.group(offset+1)
-                unit = match.group(offset+2)
-                break
-
-        if querymax is None:
-            raise FatalError("Error in shrinking! Couldn't find the max "
-                             "number of reclaimable bytes!\n"
-                             "Command output: %s" % stdout)
-
-        querymax = int(querymax)
-        if querymax == 0:
-            self.out.warn("Unable to reclaim any space. The media is full.")
-            return
-
-        # Not sure if we should use 1000 or 1024 here
-        if len(unit) == 0:  # Bytes
-            querymax /= 1000 * 1000
-        elif unit == 'K':
-            querymax /= 1000
-        elif unit == 'G':
-            querymax *= 1000
-        elif unit == 'T':
-            querymax *= 1000*1000
-
+        # Shrink the volume as much as possible and then give 100MB back.
         # From ntfsresize:
         # Practically the smallest shrunken size generally is at around
         # "used space" + (20-200 MB). Please also take into account that
         # Windows might need about 50-100 MB free space left to boot safely.
-        # I'll give 100MB extra space just to be sure
-        querymax -= 100
-
-        if querymax < 0:
-            self.out.warn("Not enough available space to shrink the image!")
-            return
-
-        self.out.output("\tReclaiming %dMB ..." % querymax)
-
         cmd = (
             r'cmd /Q /V:ON /C "SET SCRIPT=%TEMP%\QUERYMAX_%RANDOM%.TXT & ' +
             r'ECHO SELECT DISK 0 > %SCRIPT% & ' +
             'ECHO SELECT PARTITION %d >> %%SCRIPT%% & ' % self.last_part_num +
-            'ECHO SHRINK DESIRED=%d >> %%SCRIPT%% & ' % querymax +
+            r'ECHO SHRINK NOERR >> %SCRIPT% & ' +
+            r'ECHO EXTEND SIZE=100 NOERR >> %SCRIPT% & ' +
             r'ECHO EXIT >> %SCRIPT% & ' +
             r'DISKPART /S %SCRIPT% & ' +
             r'IF NOT !ERRORLEVEL! EQU 0 EXIT /B 1 & ' +
@@ -472,15 +416,17 @@ class Windows(OSBase):
                 "Shrinking failed. Please make sure the media is defragged.")
 
         for line in stdout.splitlines():
-            if line.find("%d" % querymax) >= 0:
-                self.out.output(" %s" % line)
+            line = line.strip()
+            if not len(line):
+                continue
+            self.out.info(" %s" % line)
 
         self.shrinked = True
 
     def do_sysprep(self):
         """Prepare system for image creation."""
 
-        self.out.output('Preparing system for image creation:')
+        self.out.info('Preparing system for image creation:')
 
         # Check if winexe is installed
         if not WinEXE.is_installed():
@@ -506,7 +452,7 @@ class Windows(OSBase):
         timeout = self.sysprep_params['boot_timeout'].value
         shutdown_timeout = self.sysprep_params['shutdown_timeout'].value
 
-        self.out.output("Preparing media for boot ...", False)
+        self.out.info("Preparing media for boot ...", False)
 
         with self.mount(readonly=False, silent=True):
 
@@ -514,12 +460,31 @@ class Windows(OSBase):
                 self._add_cleanup('sysprep', self.registry.reset_account,
                                   self.vm.admin.rid, False)
 
-            if self.registry.update_uac(0):
-                self._add_cleanup('sysprep', self.registry.update_uac, 1)
+            old = self.registry.update_uac(0)
+            if old != 0:
+                self._add_cleanup('sysprep', self.registry.update_uac, old)
 
-            if self.registry.update_uac_remote_setting(1):
+            old = self.registry.update_uac_remote_setting(1)
+            if old != 1:
                 self._add_cleanup('sysprep',
-                                  self.registry.update_uac_remote_setting, 0)
+                                  self.registry.update_uac_remote_setting, old)
+
+            def if_not_sysprepped(task, *args):
+                """Only perform this if the image is not sysprepped"""
+                if not self.sysprepped:
+                    task(*args)
+
+            # The next 2 registry values get completely removed by Microsoft
+            # Sysprep. They should not be reverted if Sysprep gets executed.
+            old = self.registry.update_noautoupdate(1)
+            if old != 1:
+                self._add_cleanup('sysprep', if_not_sysprepped,
+                                  self.registry.update_noautoupdate, old)
+
+            old = self.registry.update_auoptions(1)
+            if old != 1:
+                self._add_cleanup('sysprep', if_not_sysprepped,
+                                  self.registry.update_auoptions, old)
 
             # disable the firewalls
             self._add_cleanup('sysprep', self.registry.update_firewalls,
@@ -541,13 +506,13 @@ class Windows(OSBase):
         self.image.disable_guestfs()
         booted = False
         try:
-            self.out.output("Starting windows VM ...", False)
+            self.out.info("Starting windows VM ...", False)
             self.vm.start()
             try:
                 self.out.success("started (console on VNC display: %d)" %
                                  self.vm.display)
 
-                self.out.output("Waiting for OS to boot ...", False)
+                self.out.info("Waiting for OS to boot ...", False)
                 if not self.vm.wait_on_serial(timeout):
                     raise FatalError("Windows VM booting timed out!")
                 self.out.success('done')
@@ -558,17 +523,17 @@ class Windows(OSBase):
                 # conditions
                 time.sleep(5)
 
-                self.out.output("Checking connectivity to the VM ...", False)
+                self.out.info("Checking connectivity to the VM ...", False)
                 self._check_connectivity()
                 # self.out.success('done')
 
-                # self.out.output("Disabling automatic logon ...", False)
+                # self.out.info("Disabling automatic logon ...", False)
                 self._disable_autologon()
                 self.out.success('done')
 
                 self._exec_sysprep_tasks()
 
-                self.out.output("Waiting for windows to shut down ...", False)
+                self.out.info("Waiting for windows to shut down ...", False)
                 (_, stderr, rc) = self.vm.wait(shutdown_timeout)
                 if rc != 0 or "terminating on signal" in stderr:
                     raise FatalError("Windows VM died unexpectedly!\n\n"
@@ -582,7 +547,7 @@ class Windows(OSBase):
         finally:
             self.image.enable_guestfs()
 
-            self.out.output("Reverting media boot preparations ...", False)
+            self.out.info("Reverting media boot preparations ...", False)
             with self.mount(readonly=False, silent=True, fatal=False):
 
                 if not self.ismounted:
@@ -621,11 +586,11 @@ class Windows(OSBase):
         cnt = 0
         for task in enabled:
             cnt += 1
-            self.out.output(('(%d/%d)' % (cnt, size)).ljust(7), False)
+            self.out.info(('(%d/%d)' % (cnt, size)).ljust(7), False)
             task()
             del self._sysprep_tasks[task.__name__]
 
-        self.out.output("Sending shut down command ...", False)
+        self.out.info("Sending shut down command ...", False)
         if not self.sysprepped:
             self._shutdown()
         self.out.success("done")
@@ -697,8 +662,33 @@ class Windows(OSBase):
         super(Windows, self)._do_collect_metadata()
 
         # We only care for active users
-        self.meta["USERS"] = \
-            " ".join([self.usernames[a] for a in self.active_users])
+        active = [self.usernames[a] for a in self.active_users]
+        self.meta["USERS"] = " ".join(active)
+
+        # Get RDP settings
+        settings = self.registry.get_rdp_settings()
+
+        if settings['disabled']:
+            self.out.warn("RDP is disabled on the image")
+        else:
+            if 'REMOTE_CONNECTION' not in self.meta:
+                self.meta['REMOTE_CONNECTION'] = ""
+            else:
+                self.meta['REMOTE_CONNECTION'] += " "
+
+            port = settings['port']
+            if len(active):
+                rdp = ["rdp:port=%d,user=%s" % (port, user) for user in active]
+                self.meta['REMOTE_CONNECTION'] += " ".join(rdp)
+            else:
+                self.meta['REMOTE_CONNECTION'] += "rdp:port=%d" % port
+
+        major = self.image.g.inspect_get_major_version(self.root)
+        minor = self.image.g.inspect_get_minor_version(self.root)
+
+        self.meta["KERNEL"] = "Windows NT %d.%d" % (major, minor)
+        self.meta['SORTORDER'] += (100 * major + minor) * 100
+        self.meta['GUI'] = 'Windows'
 
     def _check_connectivity(self):
         """Check if winexe works on the Windows VM"""
@@ -725,10 +715,10 @@ class Windows(OSBase):
                 log.file.write("STDERR:\n%s\n" % stderr)
             finally:
                 log.close()
-            self.out.output("failed! See: `%s' for the full output" % log.name)
+            self.out.info("failed! See: `%s' for the full output" % log.name)
             if i < retries - 1:
                 wait = timeout.pop()
-                self.out.output("retrying in %d seconds ..." % wait, False)
+                self.out.info("retrying in %d seconds ..." % wait, False)
                 time.sleep(wait)
 
         raise FatalError("Connection to the Windows VM failed after %d retries"
@@ -814,12 +804,12 @@ class Windows(OSBase):
             if len(drvs) == 0:
                 del collection[drv_type]
 
-        self.out.output('Found %d valid driver%s' %
-                        (num, "s" if num != 1 else ""))
+        self.out.info('Found %d valid driver%s' %
+                      (num, "s" if num != 1 else ""))
         return collection
 
     def install_virtio_drivers(self, upgrade=True):
-        """Install new VirtIO drivers in the input media. If upgrade is True,
+        """Install new VirtIO drivers on the input media. If upgrade is True,
         then the old drivers found in the media will be removed.
         """
 
@@ -827,16 +817,44 @@ class Windows(OSBase):
         if not dirname:
             raise FatalError('No directory hosting the VirtIO drivers defined')
 
-        self.out.output('Installing VirtIO drivers:')
+        self.out.info('Installing VirtIO drivers:')
 
         valid_drvs = self._fetch_virtio_drivers(dirname)
         if not len(valid_drvs):
             self.out.warn('No suitable driver found to install!')
             return
 
+        remove = []
+        certs = []
+        install = []
+        add = []
+        # Check which drivers we need to install, which to add to the database
+        # and which to remove.
+        for dtype in valid_drvs:
+            versions = [v['DriverVer'] for k, v in valid_drvs[dtype].items()]
+            certs.extend([v['CatalogFile'] for k, v in
+                          valid_drvs[dtype].items() if 'CatalogFile' in v])
+            installed = [(k, v['DriverVer']) for k, v in
+                         self.virtio_state[dtype].items()]
+            found = [d[0] for d in installed if d[1] in versions]
+            not_found = [d[0] for d in installed if d[1] not in versions]
+
+            for drvr in found:
+                details = self.virtio_state[dtype][drvr]
+                self.out.warn('%s driver with version %s is already installed!'
+                              % (dtype, details['DriverVer']))
+            if upgrade:
+                remove.extend(not_found)
+
+            if dtype == 'viostor':
+                install.extend([d for d in valid_drvs[dtype]])
+            else:
+                add.extend([d for d in valid_drvs[dtype]])
+
         try:
-            self._upload_virtio_drivers(dirname, valid_drvs, upgrade)
-            self._boot_virtio_vm()
+            self._update_driver_database('virtio', upload=dirname, certs=certs,
+                                         add=add, install=install,
+                                         remove=remove)
         finally:
             with self.mount(readonly=False, silent=True, fatal=False):
                 if not self.ismounted:
@@ -845,31 +863,60 @@ class Windows(OSBase):
                 else:
                     self._cleanup('virtio')
 
-        self.out.output("VirtIO drivers were successfully installed")
-        self.out.output()
+        self.out.success("VirtIO drivers were successfully installed")
+        self.out.info()
 
-    def _upload_virtio_drivers(self, dirname, drvs, delete_old=True):
-        """Upload the VirtIO drivers and installation scripts to the media.
+    def _update_driver_database(self, namespace, **kwargs):
+        """Upload a directory that contains the VirtIO drivers and add scripts
+        for installing and removing specific drivers.
+
+        Keyword arguments:
+        namespace -- namespace for the cleanup entries
+        upload  -- Host directory that contains drivers to upload
+        add     -- List of drivers to add to the driver database
+        install -- List of drivers to install to the system
+        remove  -- List of drivers to remove from the system
         """
+
+        upload = kwargs['upload'] if 'upload' in kwargs else None
+        add = kwargs['add'] if 'add' in kwargs else []
+        install = kwargs['install'] if 'install' in kwargs else []
+        certs = kwargs['certs'] if 'certs' in kwargs else []
+        remove = kwargs['remove'] if 'remove' in kwargs else []
+
+        assert len(add) == 0 or upload is not None
+        assert len(install) == 0 or upload is not None
+
         with self.mount(readonly=False, silent=True):
-            # Reset Password
-            self._add_cleanup('virtio', self.registry.reset_passwd,
+            # Reset admin password
+            self._add_cleanup(namespace, self.registry.reset_passwd,
                               self.vm.admin.rid,
                               self.registry.reset_passwd(self.vm.admin.rid))
 
             # Enable admin account (if needed)
-            self._add_cleanup('virtio', self.registry.reset_account,
+            self._add_cleanup(namespace, self.registry.reset_account,
                               self.vm.admin.rid,
                               self.registry.reset_account(self.vm.admin.rid))
 
-            if self.registry.update_uac(0):
-                self._add_cleanup('virtio', self.registry.update_uac, 1)
+            old = self.registry.update_uac(0)
+            if old != 0:
+                self._add_cleanup(namespace, self.registry.update_uac, old)
+
+            old = self.registry.update_noautoupdate(1)
+            if old != 1:
+                self._add_cleanup(namespace,
+                                  self.registry.update_noautoupdate, old)
+
+            old = self.registry.update_auoptions(1)
+            if old != 1:
+                self._add_cleanup(namespace,
+                                  self.registry.update_auoptions, old)
 
             # We disable this with powershell scripts
             self.registry.enable_autologon(self.vm.admin.name)
 
             # Disable first logon animation (if needed)
-            self._add_cleanup('virtio',
+            self._add_cleanup(namespace,
                               self.registry.reset_first_logon_animation,
                               self.registry.reset_first_logon_animation(False))
 
@@ -885,35 +932,34 @@ class Windows(OSBase):
             def remove_tmp():
                 self.image.g.rm_rf("%s/%s" % (self.systemroot, tmp))
 
-            self._add_cleanup('virtio', remove_tmp)
+            self._add_cleanup(namespace, remove_tmp)
 
-            for fname in os.listdir(dirname):
-                full_path = os.path.join(dirname, fname)
-                if os.path.isfile(full_path):
-                    self.image.g.upload(
-                        full_path, "%s/%s/%s" % (self.systemroot, tmp, fname))
+            if upload is not None:
+                for fname in os.listdir(upload):
+                    full_path = os.path.join(upload, fname)
+                    if os.path.isfile(full_path):
+                        self.image.g.upload(full_path, "%s/%s/%s" %
+                                            (self.systemroot, tmp, fname))
 
             drvs_install = powershell.DRVINST_HEAD
 
-            for dtype in drvs:
-                drvs_install += "".join(
-                    [powershell.ADD_CERTIFICATE % d['CatalogFile']
-                     for d in drvs[dtype].values()])
-                cmd = powershell.ADD_DRIVER if dtype != 'viostor' \
-                    else powershell.INSTALL_DRIVER
-                drvs_install += "".join([cmd % i for i in drvs[dtype]])
+            for cert in certs:
+                drvs_install += powershell.ADD_CERTIFICATE % cert
 
-            if delete_old:
-                # Add code to remove the old drivers
-                for dtype in drvs:
-                    for oem in self.virtio_state[dtype]:
-                        drvs_install += powershell.REMOVE_DRIVER % oem
+            for driver in install:
+                drvs_install += powershell.INSTALL_DRIVER % driver
+
+            for driver in add:
+                drvs_install += powershell.ADD_DRIVER % driver
+
+            for driver in remove:
+                drvs_install += powershell.REMOVE_DRIVER % driver
 
             if self.check_version(6, 1) <= 0:
-                self._install_viostor_driver(dirname)
+                self._install_viostor_driver(upload)
                 old = self.registry.update_devices_dirs("%SystemRoot%\\" + tmp)
                 self._add_cleanup(
-                    'virtio', self.registry.update_devices_dirs, old, False)
+                    namespace, self.registry.update_devices_dirs, old, False)
                 drvs_install += powershell.DISABLE_AUTOLOGON
             else:
                 # In newer windows, in order to reduce the boot process the
@@ -944,6 +990,9 @@ class Windows(OSBase):
             # (*) to force the program to run even in Safe mode.
             self.registry.runonce({'*InstallDrivers': cmd})
 
+        # Boot the Windows VM to update the driver's database
+        self._boot_virtio_vm()
+
     def _boot_virtio_vm(self):
         """Boot the media and install the VirtIO drivers"""
 
@@ -953,7 +1002,7 @@ class Windows(OSBase):
             timeout = self.sysprep_params['boot_timeout'].value
             shutdown_timeout = self.sysprep_params['shutdown_timeout'].value
             virtio_timeout = self.sysprep_params['virtio_timeout'].value
-            self.out.output("Starting Windows VM ...", False)
+            self.out.info("Starting Windows VM ...", False)
             booted = False
             try:
                 if old_windows:
@@ -965,16 +1014,16 @@ class Windows(OSBase):
 
                 self.out.success("started (console on VNC display: %d)" %
                                  self.vm.display)
-                self.out.output("Waiting for Windows to boot ...", False)
+                self.out.info("Waiting for Windows to boot ...", False)
                 if not self.vm.wait_on_serial(timeout):
                     raise FatalError("Windows VM booting timed out!")
                 self.out.success('done')
                 booted = True
-                self.out.output("Installing new drivers ...", False)
+                self.out.info("Installing new drivers ...", False)
                 if not self.vm.wait_on_serial(virtio_timeout):
                     raise FatalError("Windows VirtIO installation timed out!")
                 self.out.success('done')
-                self.out.output('Shutting down ...', False)
+                self.out.info('Shutting down ...', False)
                 (_, stderr, rc) = self.vm.wait(shutdown_timeout)
                 if rc != 0 or "terminating on signal" in stderr:
                     raise FatalError("Windows VM died unexpectedly!\n\n"
@@ -996,7 +1045,7 @@ class Windows(OSBase):
             # Hopefully restart in safe mode. Newer windows will not boot from
             # a viostor device unless we initially start them in safe mode
             try:
-                self.out.output('Rebooting Windows VM in safe mode ...', False)
+                self.out.info('Rebooting Windows VM in safe mode ...', False)
                 self.vm.start()
                 (_, stderr, rc) = self.vm.wait(timeout + shutdown_timeout)
                 if rc != 0 or "terminating on signal" in stderr:
